package com.mall.product.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.mall.common.domain.vo.result.PageResult;
import com.mall.product.domain.dto.BrandDto;
import com.mall.product.domain.dto.CategoryBrandRelationDto;
import com.mall.product.domain.entity.BrandEntity;
import com.mall.product.domain.vo.BrandVo;
import com.mall.product.mapper.BrandMapper;
import com.mall.product.mapper.CategoryBrandRelationMapper;
import com.mall.product.service.BrandService;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * <p>
 * 品牌 服务实现类
 * </p>
 *
 * @author Bunny
 * @since 2025-07-04 14:05:53
 */
@Service
@Transactional
@RequiredArgsConstructor
public class BrandServiceImpl extends ServiceImpl<BrandMapper, BrandEntity> implements BrandService {

    private final CategoryBrandRelationMapper categoryBrandRelationMapper;

    /**
     * * 品牌 服务实现类
     *
     * @param pageParams 品牌分页查询page对象
     * @param dto        品牌分页查询对象
     * @return 查询分页品牌返回对象
     */
    @Override
    public PageResult<BrandVo> getBrandPage(Page<BrandEntity> pageParams, BrandDto dto) {
        IPage<BrandVo> page = baseMapper.selectListByPage(pageParams, dto);

        return PageResult.<BrandVo>builder()
                .list(page.getRecords())
                .pageNo(page.getCurrent())
                .pageSize(page.getSize())
                .total(page.getTotal())
                .build();
    }

    /**
     * 添加品牌
     *
     * @param dto 品牌添加
     */
    @Override
    public void addBrand(BrandDto dto) {
        BrandEntity brandEntity = new BrandEntity();
        BeanUtils.copyProperties(dto, brandEntity);
        save(brandEntity);
    }

    /**
     * 更新品牌
     * 先更新自己的品牌数据
     * 之后更新冗余表中的数据
     *
     * @param dto {@link BrandDto} 更新表单
     */
    @Override
    public void updateBrand(BrandDto dto) {
        BrandEntity brandEntity = new BrandEntity();
        BeanUtils.copyProperties(dto, brandEntity);
        updateById(brandEntity);

        // 品牌名不为空保存品牌名
        String name = brandEntity.getName();
        if (StringUtils.isNotBlank(name)) {
            // 根据品牌id更新 品牌分类关系表
            CategoryBrandRelationDto categoryBrandRelationDto = CategoryBrandRelationDto.builder()
                    .brandId(dto.getBrandId())
                    .brandName(name)
                    .build();
            categoryBrandRelationMapper.updateByBrandIdAndCatId(categoryBrandRelationDto);
        }
    }

    /**
     * 删除|批量删除品牌
     *
     * @param ids 删除id列表
     */
    @Override
    public void deleteBrand(List<Long> ids) {
        removeByIds(ids);
    }
}